cpdef _create_igraph(nodes, edges, from_id_col, to_id_col, node_id_col)
cpdef _create_nxgraph(nodes, edges, from_id_col, to_id_col, node_id_col)
cpdef _create_pdgraph(nodes, edges, from_id_col, to_id_col, weight_cols)
cpdef generate_directed_edges(edges, direction, from_id_col, to_id_col, force_bidirectional)

